from django.db import models
from django.contrib.auth.models import User


class RelVisitor(models.Model):
    user = models.OneToOneField(User)
    
    visitor_id = models.CharField(max_length=64)




from datetime import datetime
import uuid

from hoarder.models import RelVisitor
from hoarder import tasks
from hoarder.backends import get_sync_backends

def _get_visitor_id(request):
    if 'visitor_id' in request.session:
        visitor_id = request.session['visitor_id']
        if type(visitor_id) != str:
            visitor_id = str(visitor_id)
            request.session['visitor_id'] = visitor_id
    else:
        visitor_id = str(uuid.uuid4())
        tasks.create_visitor.delay(visitor_id)
        for b in get_sync_backends(request):
            b.create_visitor(visitor_id)
        request.session['visitor_id'] = visitor_id
    return visitor_id
    

def register_request_event(request, 
                           event_type, 
                           event_data={},
                           when=None):
    tasks.register_event.delay(event_type,
                               _get_visitor_id(request),
                               when=when,
                               data=event_data)
    for b in get_sync_backends(request):
        b.register_event(event_type,
                         _get_visitor_id(request),
                         when=when,
                         data=event_data)

    

def register_user_event(user, 
                        event_type, 
                        event_data={},
                        when=None):
    try:
        visitor_id = user.relvisitor.visitor_id
    except RelVisitor.DoesNotExist:
        visitor_id = user.id
    tasks.register_event.delay(event_type,
                               visitor_id,
                               when=when,
                               data=event_data)
    


def label_visitor(request, label):
    tasks.label_visitor.delay(_get_visitor_id(request),
                              label)
    for b in get_sync_backends(request):
        b.label_visitor(_get_visitor_id(request),
                        label)
        



from pymongo import Connection
from datetime import datetime

from django.conf import settings

connection = Connection(getattr(settings, 'HOARDER_MONGO_HOST', 'localhost'), 
                        getattr(settings, 'HOARDER_MONGO_PORT', 27017))

db = connection[getattr(settings, 'HOARDER_MONGO_DATABASE', 'hoarder')]

visitors = db.visitors
events = db.events


class MongoBackend(object):

    is_async = True

    def get_context(self, request):
        return {}

    def register_event(self,
                       event_type,
                       visitor_id,
                       when=None,
                       data={}):
        if when is None:
            when = datetime.now()
        events.insert({'type' : event_type,
                       'timestamp' : when,
                       'visitor_id' : visitor_id,
                       'data' : data})
                       
    def label_visitor(self, 
                      visitor_id,
                      label):
        visitor = visitors.find_one({'id' : visitor_id})
        labels = visitor.get('labels', [])
        labels.append(label)
        visitor['labels'] = labels
        visitors.save(visitor)
                           

    def create_visitor(self,
                       visitor_id):
        visitors.insert({'id' : visitor_id})
                      
    def deduplicate(self,
                    from_visitor_id,
                    to_visitor_id):
        events.update({'visitor_id' : from_visitor_id },
                      {'$set' : {'visitor_id' : to_visitor_id }},
                      multi=True)
        old_visitor = visitors.find_one({'id' : from_visitor_id})
        new_visitor = visitors.find_one({'id' : to_visitor_id})
        labels = new_visitor.get('labels', [])
        labels += old_visitor.get('labels',[])
        new_visitor['labels'] = labels
        visitors.save(new_visitor)
        visitors.remove({'id': from_visitor_id})
        
    def set_user(self,
                 visitor_id,
                 user):
        pass

    def get_tracking_code(self,
                          context):
        return ''



import re
from datetime import datetime
import uuid

from django.conf import settings

from hoarder.models import RelVisitor
from hoarder import register_request_event, _get_visitor_id
from hoarder.tasks import deduplicate, set_user

RE_HEADERS = re.compile('^HTTP_')


def _flatten_type(val):
    if isinstance(val, str) or isinstance(val, unicode) or isinstance(val, long) or isinstance(val, int):
        return val
    else:
        return unicode(val)

class LoggingMiddleware(object):

    def _get_headers(self, request):
        return dict((RE_HEADERS.sub('', header), value) for (header, value) 
                    in request.META.items() if header.startswith('HTTP_'))

    def process_request(self, request):
        visitor_id = _get_visitor_id(request)
        request_log = {'headers' : self._get_headers(request),
                       'path' : request.path,
                       'method' : request.method,
                       'get' : dict(request.GET)}
        register_request_event(request,
                               'request',
                               request_log)

            
        
    def process_view(self, request, view_func, view_args, view_kwargs):
        args = [_flatten_type(x) for x in view_args]
        kwargs = dict([(x,_flatten_type(y)) for x,y in view_kwargs.items()])
        view = {'view' : view_func.__module__ + '.' + view_func.__name__ ,
                'args' : args,
                'kwargs' : kwargs}
        register_request_event(request, 'view', view)
        
        

class DeduplicationMiddleware(object):
    def process_request(self, request):
        visitor_id = _get_visitor_id(request)
        if request.user.is_authenticated():
            try:
                rv = request.user.relvisitor
                if rv.visitor_id != visitor_id:
                    deduplicate.delay(visitor_id, rv.visitor_id)
                    request.session['visitor_id'] = rv.visitor_id
            except RelVisitor.DoesNotExist:
                RelVisitor.objects.create(user=request.user,
                                          visitor_id=visitor_id)
                set_user.delay(visitor_id,
                               request.user.id)
        



from django.conf import settings
from django.contrib.auth.models import User

from celery.decorators import task

from hoarder.backends import get_async_backends
    

@task
def create_visitor(visitor_id):
    for backend in get_async_backends():
        backend.create_visitor(visitor_id)


@task
def register_event(event_type,
                   visitor_id,
                   when,
                   data):    
    for backend in get_async_backends():
        backend.register_event(event_type,
                               visitor_id,
                               when,
                               data)


@task
def label_visitor(visitor_id,
                  label):    
    for backend in get_async_backends():
        backend.label_visitor(visitor_id,
                              label)


@task
def deduplicate(from_visitor_id, to_visitor_id):
    for backend in get_async_backends():
        backend.deduplicate(from_visitor_id,
                            to_visitor_id)

    return True


@task
def set_user(visitor_id, user_id):
    user = User.objects.get(id=user_id)
    for backend in get_async_backends():
        backend.set_user(visitor_id,
                         user)
    



import time
import logging
from datetime import datetime

from django.conf import settings
from django.utils.importlib import import_module
from django.template.loader import render_to_string

_backends = None

def get_backends():
    global _backends
    if _backends is None:
        _backends = []
        for backend_path in settings.HOARDER_BACKENDS:
            be_module, be_classname = backend_path.rsplit('.', 1)
            mod = import_module(be_module)
            be_class = getattr(mod, be_classname)
            _backends.append(be_class())
    return _backends


def get_sync_backends(request):
    for backend_path in settings.HOARDER_BACKENDS:
        be_module, be_classname = backend_path.rsplit('.', 1)
        mod = import_module(be_module)
        be_class = getattr(mod, be_classname)
        if not be_class.is_async:
            yield be_class(request)
    

def get_async_backends():
    for b in get_backends():
        if b.is_async:
            yield b
    
        
class KISSMetricsBackend(object):
    is_async = True

    def get_context(self, request):
        if not request.session.get('identify_kiss'):
            identify_kiss = True
            request.session['identify_kiss'] = True
        else:
            identify_kiss = False
        return {'identify_kiss' : identify_kiss,
                'KISSMETRICS_API_KEY' : settings.KISSMETRICS_API_KEY}


    def create_visitor(self,
                       visitor_id):
        pass

    def register_event(self,
                       event_type,
                       visitor_id,
                       when=None,
                       data={}):
        if event_type in ('request', 'view'):
            return
        from KISSmetrics import KM
        km = KM(settings.KISSMETRICS_API_KEY)
        km.identify(visitor_id)
        if when:
            data['_d'] = 1
            data['_t'] = int(time.mktime(when.timetuple()))
        km.record(event_type, data)
        

    def label_visitor(self,
                      visitor_id,
                      label):
        from KISSmetrics import KM
        km = KM(settings.KISSMETRICS_API_KEY)
        km.identify(visitor_id)
        km.set({label:1})

    def deduplicate(self,
                    from_visitor_id,
                    to_visitor_id):
        from KISSmetrics import KM
        km = KM(settings.KISSMETRICS_API_KEY)
        km.alias(from_visitor_id, to_visitor_id)

    def set_user(self,
                 visitor_id,
                 user):
        from KISSmetrics import KM
        km = KM(settings.KISSMETRICS_API_KEY)
        km.identify(visitor_id)
        km.set({'user_id' : user.id, 'name' : user.get_full_name() })

    def get_tracking_code(self,
                          context):
        if not context['request'].session.get('identify_kiss'):
            identify_kiss = True
            request.session['identify_kiss'] = True
        else:
            identify_kiss = False
        
        return render_to_string('hoarder/km.html',
                                {'identify_kiss' : identify_kiss,
                                 'KISSMETRICS_API_KEY' : settings.KISSMETRICS_API_KEY},
                                context_instance=context)


class LogBackend(object):
    is_async = False
    
    def __init__(self, request=None):
        self.logger = logging.getLogger('hoarder')

    def get_context(self, request):
        return {}

    def create_visitor(self,
                       visitor_id):
        self.logger.info('create_visitor(%s)'%repr(visitor_id))

    def label_visitor(self,
                       visitor_id):
        self.logger.info('label_visitor(%s, %s)'%(repr(visitor_id), repr(label)))
        
    def register_event(self,
                       event_type,
                       visitor_id,
                       when=None,
                       data={}):
    
        self.logger.info('register_event(%s,%s,%s,%s)'%(repr(event_type),
                                                        repr(visitor_id),
                                                        repr(when),
                                                        repr(data)))
    def deduplicate(self,
                    from_visitor_id,
                    to_visitor_id):
        self.logger.info('deduplicate(%s, %s)'%(repr(from_visitor_id),
                                               repr(to_visitor_id)))

    def set_user(self,
                 visitor_id,
                 user):
        self.logger.info('set_user(%s, %s)'%(repr(visitor_id),
                                             repr(user)))

    def get_tracking_code(self,
                          context):
        return ''



from datetime import datetime

from django.conf import settings
from django.template.loader import render_to_string

class GABackend(object):

    is_async = False

    def __init__(self, request=None):
        self.request = request

    def register_event(self,
                       event_type,
                       visitor_id,
                       when=None,
                       data={}):
        if not self.request:
            return 
        if event_type in ('request', 'view'):
            return
        if not hasattr(self.request, 'ga_events'):
            self.request.ga_events = []
        self.request.ga_events.append({'type' : event_type,
                                       'timestamp' : when,
                                       'visitor_id' : visitor_id,
                                       'data' : data})
                       
    def label_visitor(self, 
                      visitor_id,
                      label):
        pass

    def create_visitor(self,
                       visitor_id):
        pass

                      
    def deduplicate(self,
                    from_visitor_id,
                    to_visitor_id):
        pass

    def set_user(self,
                 visitor_id,
                 user):
        pass

    def get_tracking_code(self,
                          context):
        return render_to_string('hoarder/ga.html',
                                {'GA_PROPERTY_ID' : settings.GA_PROPERTY_ID},
                                context_instance=context)

